/* This file was generated automatically by the Snowball to ISO C compiler */
/* http://snowballstem.org/ */

#ifdef __cplusplus
extern "C" {
#endif

extern struct SN_env * romanian_UTF_8_create_env(void);
extern void romanian_UTF_8_close_env(struct SN_env * z);

extern int romanian_UTF_8_stem(struct SN_env * z);

#ifdef __cplusplus
}
#endif

